package de.fraunhofer.iem.app.kpi.service

import de.fraunhofer.iem.app.kpi.dto.KPITreeChildResponseDto
import de.fraunhofer.iem.app.kpi.dto.KPITreeResponseDto
import de.fraunhofer.iem.app.logger.getLogger
import de.fraunhofer.iem.app.repository.dto.RepositoryDetailsDto
import de.fraunhofer.iem.app.tools.occmd.enumeration.Checks
import de.fraunhofer.iem.app.tools.occmd.json.RawResultJson
import de.fraunhofer.iem.kpiCalculator.core.KpiCalculator
import de.fraunhofer.iem.kpiCalculator.model.kpi.KpiId
import de.fraunhofer.iem.kpiCalculator.model.kpi.RawValueKpi
import de.fraunhofer.iem.kpiCalculator.model.kpi.hierarchy.DefaultHierarchy
import de.fraunhofer.iem.kpiCalculator.model.kpi.hierarchy.KpiResultHierarchy
import org.springframework.stereotype.Service

@Service
class KPIService {

    private val logger = getLogger(javaClass)

    fun calculateKpiHierarchy(rawValueKpis: List<RawValueKpi>): KpiResultHierarchy {
        return KpiCalculator.calculateKpis(DefaultHierarchy.get(), rawValueKpis)
    }

    fun removeKPIChildrenLowerThanSecondLevel(kpiTreeResponseDto: KPITreeResponseDto): KPITreeResponseDto {
        val children: List<KPITreeChildResponseDto> = kpiTreeResponseDto.children.map {
            KPITreeChildResponseDto(
                kpi = KPITreeResponseDto(
                    value = it.kpi.value,
                    name = it.kpi.name,
                    description = it.kpi.description,
                    isRoot = it.kpi.isRoot,
                    displayValue = it.kpi.displayValue,
                    emptyList(),
                    it.kpi.isEmpty,
                    order = it.kpi.order
                ),
                actualWeight = it.actualWeight,
                plannedWeight = it.plannedWeight
            )
        }

        return KPITreeResponseDto(
            value = kpiTreeResponseDto.value,
            name = kpiTreeResponseDto.name,
            description = kpiTreeResponseDto.description,
            isRoot = true,
            displayValue = kpiTreeResponseDto.displayValue,
            children = children,
            isEmpty = kpiTreeResponseDto.isEmpty,
            order = kpiTreeResponseDto.order
        )
    }

    /**
     * This method calculates the OccmdCreateKpiDtos based upon the given tool results.
     */
    fun calculateOccmdKpis(rawOccmdResults: List<RawResultJson>): List<RawValueKpi> {
        val kpis = mutableListOf<RawValueKpi>()

        rawOccmdResults.forEach {
            when (val check = Checks.fromString(it.check)) {
                Checks.CheckedInBinaries ->
                    kpis.add(
                        RawValueKpi(
                            kind = KpiId.CHECKED_IN_BINARIES,
                            score = (it.score * 100).toInt()
                        )
                    )

                Checks.SastUsageBasic ->
                    kpis.add(
                        RawValueKpi(
                            kind = KpiId.SAST_USAGE,
                            score = (it.score * 100).toInt()
                        )
                    )

                Checks.Secrets ->
                    kpis.add(
                        RawValueKpi(
                            kind = KpiId.SECRETS,
                            score = (it.score * 100).toInt()
                        )
                    )

                Checks.CommentsInCode ->
                    kpis.add(
                        RawValueKpi(
                            kind = KpiId.COMMENTS_IN_CODE,
                            score = (it.score * 100).toInt()
                        )
                    )

                Checks.DocumentationInfrastructure ->
                    kpis.add(
                        RawValueKpi(
                            kind = KpiId.DOCUMENTATION_INFRASTRUCTURE,
                            score = (it.score * 100).toInt()
                        )
                    )

                else ->
                    logger.warn("Unknown OCCMD result. $check")
            }
        }

        return kpis
    }

    /**
     * Creates a named map of RepositoryCreateDtos, based upon the provided repository details.
     * This method only returns raw KPIs.
     */
    fun calculateRepositoryDetailsKpis(repoDetailsDto: RepositoryDetailsDto): List<RawValueKpi> {
        return listOf(
            RawValueKpi(
                kind = KpiId.NUMBER_OF_COMMITS,
                score = repoDetailsDto.numberOfCommits
            ),
            RawValueKpi(
                kind = KpiId.NUMBER_OF_SIGNED_COMMITS,
                score = repoDetailsDto.numberOfSignedCommits
            ),
            RawValueKpi(
                kind = KpiId.IS_DEFAULT_BRANCH_PROTECTED,
                score = if (repoDetailsDto.isDefaultBranchProtected) 100 else 0
            )
        )
    }


}
